from typing import cast

from pysmscenter.exceptions import TwoFactorExceptionError
from pysmscenter.types import TwoFactorCheckResponse, TwoFactorRawResponse
from pysmscenter.utils import bool2str, raise_for_errors

from .manager import Manager


class TwoFactorManager(Manager):
    name = "two_factor"

    def __str__(self) -> str:
        return self.__class__.__name__

    def send(
        self,
        to: str,
        text: str | None = None,
        sender: str | None = None,
        wait: int | None = None,
        callback: str | None = None,
        ucs: bool | None = None,
    ) -> TwoFactorRawResponse:
        """
        Send a 2FA code to a mobile number.

        Args:
            to (str): The recipient's mobile number.
            text (str, optional): The text message to send. If not provided, a default message will be used.
            sender (str, optional): The sender ID to use for the message.
            If not provided, the default sender will be used.
            wait (int, optional): The time in seconds to wait for a response.
            If not provided, the default wait time will be used.
            callback (str, optional): A URL to receive delivery reports.
            If not provided, no delivery reports will be sent.
            ucs (bool, optional): Whether to send the message in UCS-2 encoding.
            If not provided, the default encoding will be used.

            Extra:
            Both wait and callback paramters are missing then the call will wait for 10 seconds
            by default for the delivery report.
            If the callback parameter is set, or the wait parameter is 0,
            the call will return immediately and if callback is set it will be called
            as soon as the delivery report arrives.
        """

        params = {
            "to": to,
            "text": text,
            "from": sender,
            "wait": wait,
            "callback": callback,
            "ucs": bool2str(ucs) if ucs is not None else None,
        }

        params = {key: value for key, value in params.items() if value is not None}

        response = self.call("GET", "2fa/send", params=params)
        raise_for_errors(response, TwoFactorExceptionError)
        return cast(TwoFactorRawResponse, response)

    def check(self, auth_id: str, code: str) -> TwoFactorCheckResponse:
        """
        Check the 2FA code for a given auth_id.

        Args:
            auth_id (str): The authentication ID received when sending the 2FA code.
            code (str): The 2FA code to check.
        """

        params = {
            "authId": auth_id,
            "code": code,
        }

        response = self.call("GET", "2fa/check", params=params)
        raise_for_errors(response, TwoFactorExceptionError)
        return cast(TwoFactorCheckResponse, response)



from datetime import date
from typing import Any, cast

from pysmscenter.exceptions import ContactExceptionError
from pysmscenter.types import BaseResponse, ContactData, ContactDetail, ContactListData, DateLike
from pysmscenter.utils import parse_date, raise_for_errors

from .manager import Manager


class ContactManager(Manager):
    name = "contact"

    def __str__(self) -> str:
        return self.__class__.__name__

    def add(
        self,
        mobile: str,
        name: str = "",
        surname: str = "",
        full_name: str = "",
        vname: str = "",
        vusername: str = "",
        birthday: DateLike | None = None,
        nameday: DateLike | None = None,
        **kwargs: Any,
    ) -> ContactData:
        """Add a contact.

        Args:
            mobile (str): Mobile number of the contact
            name (str): Name of the contact
            surname (str): Surname of the contact
            full_name (str | None, optional): Fullname of contact. Defaults to None.
            vname (str | None, optional): First name in vocative. Useful for personalised messages. Defaults to None.
            vusername (str | None, optional): Last name in vocative. Useful for personalised messages. Defaults to None
            birthday (DateLike | None, optional): Birthday of contact in YYYY-MM-DD format. Defaults to None.
            nameday (DateLike | None, optional): Nameday of contact in YYYY-MM-DD format. Defaults to None.
            **kwargs: Additional API parameters to include.

        Raises:
            ContactExceptionError: If the API response indicates an error.

        Returns:
           ContactData: Response from the API.
        """
        params = {
            "mobile": mobile,
            "name": name,
            "surname": surname,
            "full_name": full_name,
            "vname": vname,
            "vusername": vusername,
            "birthday": self._date_to_api(birthday),
            "nameday": self._date_to_api(nameday),
            **kwargs,
        }

        response = self.call("GET", "contact/add", params)

        raise_for_errors(response, ContactExceptionError)

        return cast(ContactData, response)

    def list(self) -> ContactListData:
        """Get the list of contacts.

        Raises:
            ContactExceptionError: If the API response indicates an error.

        Returns:
            ContactListData: Response from the API.
        """
        response = self.call("GET", "contact/list")
        return cast(ContactListData, response)

    def get(self, contact_id: str) -> ContactDetail:
        """Get a contact's details.

        Args:
            contact_id: Contact ID to retrieve.

        Raises:
            ContactExceptionError: If the API response indicates an error.

        Returns:
            ContactDetail: Response from the API.
        """
        response = self.call("GET", "contact/get", {"contactId": contact_id})
        raise_for_errors(response, ContactExceptionError)
        return cast(ContactDetail, response)

    def delete(self, contact_id: str) -> BaseResponse:
        """Delete a contact.

        Args:
            contact_id (str): Contact ID of the contact to delete.

        Raises:
            ContactExceptionError: If the API response indicates an error.

        Returns:
            BaseResponse: Response from the API.
        """

        response = self.call("GET", "contact/delete", {"contactId": contact_id})

        raise_for_errors(response, ContactExceptionError)

        return cast(BaseResponse, response)

    def update(
        self,
        contact_id: str,
        mobile: str | None = None,
        name: str | None = None,
        surname: str | None = None,
        full_name: str | None = None,
        vname: str | None = None,
        vusername: str | None = None,
        birthday: DateLike | None = None,
        nameday: DateLike | None = None,
        **kwargs: Any,
    ) -> ContactData:
        """Update a contact.

        Args:
            contact_id (str): Contact ID to update
            mobile (str | None, optional): Mobile number of the contact
            name (str | None, optional): Name of the contact
            surname (str | None, optional): Surname of the contact
            full_name (str | None, optional): Fullname of contact. Defaults to None.
            vname (str | None, optional): First name in vocative. Useful for personalised messages. Defaults to None.
            vusername (str | None, optional): Last name in vocative. Useful for personalised messages. Defaults to None
            birthday (DateLike | None, optional): Birthday of contact in YYYY-MM-DD format. Defaults to None.
            nameday (DateLike | None, optional): Nameday of contact in YYYY-MM-DD format. Defaults to None.
            **kwargs: Additional API parameters to include.

        Raises:
            ContactExceptionError: If the API response indicates an error.

        Returns:
            ContactData: Response from the API.
        """

        params = {
            "contactId": contact_id,
            "mobile": mobile,
            "name": name,
            "surname": surname,
            "full_name": full_name,
            "vname": vname,
            "vusername": vusername,
            "birthday": self._date_to_api(birthday),
            "nameday": self._date_to_api(nameday),
            **kwargs,
        }
        params = {key: value for key, value in params.items() if value is not None}
        response = self.call("GET", "contact/update", params)

        raise_for_errors(response, ContactExceptionError)

        return cast(ContactData, response)

    @staticmethod
    def _date_to_api(value: date | str | None) -> str | None:
        """Convert a date-like value to the API's date string.

        Args:
            value: Date or ISO-like string.

        Returns:
            ISO-8601 date string, or None if value is None.

        Raises:
            ContactExceptionError: If the date string cannot be parsed.
        """
        if value is None:
            return None

        if isinstance(value, date):
            return value.isoformat()
        try:
            return parse_date(value).isoformat()
        except ValueError as exc:
            raise ContactExceptionError(str(exc)) from exc



from typing import cast

from email_validator import EmailNotValidError, validate_email

from pysmscenter.exceptions import SMSClientError, UserCommentExceptionError, UserExceptionError
from pysmscenter.types import (
    BaseResponse,
    UserCommentListRawResponseType,
    UserCommentRawResponse,
    UserListRawResponseType,
    UserRawResponse,
)
from pysmscenter.utils import raise_for_errors

from .manager import Manager


class UserManager(Manager):
    name = "user"

    def __str__(self) -> str:
        return self.__class__.__name__

    def add(self, email: str, password: str) -> UserRawResponse:
        """
        Add a new sub-account with the given email and password.

        Args:
            email (str): The email address for the new sub-account.
            password (str): The password for the new sub-account.

        """

        self._validate_email(email)
        params = {"email": email, "password": password}
        response = self.call("GET", "user/add", params=params)
        raise_for_errors(response, UserExceptionError)
        return cast(UserRawResponse, response)

    def delete(self, user_id: str) -> None:
        """
        Delete a sub-account by its user ID.

        Args:
            user_id (str): The ID of the sub-account to delete.
        """
        raise NotImplementedError("The delete user endpoint is not implemented in the API documentation.")

    def email(self) -> None:
        raise NotImplementedError("The email user endpoint is not implemented in the API documentation.")

    def email_all(self) -> None:
        raise NotImplementedError("The email all users endpoint is not implemented in the API documentation.")

    def get(self) -> None:
        raise NotImplementedError("The get user endpoint is not implemented in the API documentation.")

    def sms(self) -> None:
        raise NotImplementedError("The get user SMS endpoint is not implemented in the API documentation.")

    def sms_all(self) -> None:
        raise NotImplementedError("The get all users SMS endpoint is not implemented in the API documentation.")

    def update(self) -> None:
        raise NotImplementedError("The update user endpoint is not implemented in the API documentation.")

    def list(self) -> UserListRawResponseType:
        """
        List all sub-accounts under the main account.

        Returns:
            UserListRawResponseType: The response containing the list of sub-accounts.
        """
        response = self.call("GET", "user/list")
        return cast(UserListRawResponseType, response)

    def topup(self, user_id: str, sms: str, cost: str) -> UserRawResponse:
        """
        Top up a sub-account with the specified amount.

        Args:
            user_id (str): The ID of the sub-account to top up.
            sms (str): The number of SMS to add to the sub-account balance.
            cost (str): The cost associated with the top-up.

        Returns:
            UserRawResponse: The response containing the updated sub-account information.
        """
        params = {"userId": user_id, "sms": sms, "cost": cost}
        response = self.call("GET", "user/topup", params=params)
        raise_for_errors(response, UserExceptionError)
        return cast(UserRawResponse, response)

    def add_comment(self, user_id: str, comment: str) -> UserCommentRawResponse:
        """
        Add a comment to a sub-account.

        Args:
            user_id (str): The ID of the sub-account to add a comment to.
            comment (str): The comment to add to the sub-account.
        """
        params = {"userId": user_id, "comment": comment}
        response = self.call("GET", "user/comment/add", params=params)
        raise_for_errors(response, UserCommentExceptionError)
        return cast(UserCommentRawResponse, response)

    def delete_comment(self, comment_id: str) -> BaseResponse:
        """
        Delete a comment from a sub-account.

        Args:
            comment_id (str): The ID of the comment to delete.
        """
        params = {"commentId": comment_id}
        response = self.call("GET", "user/comment/delete", params=params)
        raise_for_errors(response, SMSClientError)
        return cast(BaseResponse, response)

    def comments(self, user_id: str) -> UserCommentListRawResponseType:
        """
        List all comments for a sub-account.

        Args:
            user_id (str): The ID of the sub-account to list comments for.
        """
        params = {"userId": user_id}
        response = self.call("GET", "user/comment/list", params=params)
        raise_for_errors(response, SMSClientError)
        return cast(UserCommentListRawResponseType, response)

    @staticmethod
    def _validate_email(email: str) -> None:
        """
        Validate an email address.

        Args:
            email (str): The email address to validate.

        Returns:
            None
        """
        try:
            validate_email(email)
        except EmailNotValidError as e:
            raise ValueError(f"Invalid email address: {email}") from e



from .base import BaseResponse


class HLRLookupRawResponse(BaseResponse, total=False):
    result: str
    description: str
    mcc: str
    mnc: str
    network: str
    country: str
    countryCode: str  # noqa: N815
    ported: str
    cctld: str
    mccInitial: str  # noqa: N815
    mncInitial: str  # noqa: N815



from datetime import date
from typing import TypedDict

from pysmscenter.types.base import BaseResponse

type DateLike = str | date


class ContactID(TypedDict):
    contactId: str


class ContactData(BaseResponse, total=False):
    contact: ContactID


class Contacts(TypedDict):
    contactId: str
    mobile: str
    smscost: str
    name: str
    surname: str
    vname: str
    birthday: str
    nameday: str
    custom1: str
    custom2: str


class ContactListData(BaseResponse, total=False):
    total: str
    contacts: list[Contacts]


class Contact(TypedDict):
    contactId: str
    mobile: str
    smscost: str
    name: str
    surname: str
    vname: str
    birthday: str
    nameday: str


class ContactDetail(BaseResponse, total=False):
    total: str
    contact: Contact



from typing import TypedDict

from pysmscenter.types.base import BaseResponse


class SmS(TypedDict, total=False):
    smsId: str
    sender: str
    flash: str
    unicode: str
    to: str
    text: str
    timestamp: str
    status: str
    cost: str
    ttd: str
    contactId: str


class SingleListHistoryRawData(BaseResponse, total=False):
    total: str
    sms: list[SmS]


class HistoryGroupSms(TypedDict, total=False):
    smsId: str
    contactId: str
    to: str
    status: str
    cost: str
    ttd: str


class HistoryGroupItem(TypedDict, total=False):
    groupId: str
    sender: str
    flash: str
    unicode: str
    timestamp: str
    text: str
    total: str
    cost: str
    sms: list[HistoryGroupSms]


type GroupListHistoryRawResponse = dict[str, HistoryGroupItem | str]



from typing import TypedDict

from .base import BaseResponse


class UserItem(TypedDict):
    userId: str
    balance: int


class UserRawResponse(BaseResponse, total=False):
    user: UserItem


class UserListItem(TypedDict, total=False):
    userId: str
    username: str
    email: str
    balance: str
    mobile: str
    key: str | None


class UserListRawResponse(BaseResponse, total=False):
    total: str


type UserListRawResponseType = dict[str, UserListItem | str]


class CommentItem(TypedDict):
    commentId: str


class UserCommentRawResponse(BaseResponse, total=False):
    comment: CommentItem


class UserCommentListItem(TypedDict):
    commentId: str
    comment: str
    timestamp: str


class UserCommentListRawResponseType(BaseResponse, total=False):
    total: str
    comments: list[UserCommentListItem]



from typing import cast

from pysmscenter.exceptions import MobileExceptionError
from pysmscenter.types import MobileRawData
from pysmscenter.utils import raise_for_errors

from .manager import Manager


class MobileManager(Manager):
    name = "mobile"

    def __str__(self) -> str:
        return self.__class__.__name__

    def check(self, mobile: str) -> MobileRawData:
        """Check a mobile number.

        Args:
            mobile (str): Mobile number to check

        Raises:
            MobileExceptionError: If the API response indicates an error.

        Returns:
            MobileRawData: Response from the API.
        """
        params = {"mobile": mobile}
        response = self.call("GET", "mobile/check", params)
        raise_for_errors(response, MobileExceptionError)

        return cast(MobileRawData, response)



from typing import TypedDict

from pysmscenter.types.base import BaseResponse


class Group(TypedDict):
    groupId: str


class GroupList(TypedDict):
    groupId: str
    name: str


class GroupData(BaseResponse, total=False):
    group: Group


class GroupListData(BaseResponse, total=False):
    total: str
    groups: list[GroupList]


class GroupContact(TypedDict, total=False):
    contactId: str
    mobile: str
    smscost: str
    name: str
    surname: str
    vname: str
    birthday: str
    nameday: str
    custom1: str
    custom2: str


class GroupGetGroup(TypedDict, total=False):
    name: str
    total: str
    contacts: list[GroupContact]


class GroupGetData(BaseResponse, total=False):
    total: str
    group: GroupGetGroup


class GroupContactLink(TypedDict):
    contactGroupId: str


class GroupAddContactGroup(TypedDict):
    contact: GroupContactLink


class GroupAddContactData(BaseResponse, total=False):
    group: GroupAddContactGroup



import typing


class SMSClientError(Exception):
    def __init__(
        self, message: str | None = None, code: str | None = None, response: dict[str, typing.Any] | None = None
    ) -> None:
        self.message = message
        self.code = code
        self.response = response
        super().__init__(message)

    def __str__(self) -> str:
        msg = super().__str__()
        return f"[{self.code}] {msg}" if self.code and msg else (f"[{self.code}]" if self.code else msg)


class CredentialError(SMSClientError):
    pass


class SMSExceptionError(SMSClientError):
    pass


class ContactExceptionError(SMSClientError):
    pass


class MobileExceptionError(SMSClientError):
    pass


class GroupExceptionError(SMSClientError):
    pass


class HLRExceptionError(SMSClientError):
    pass


class TwoFactorExceptionError(SMSClientError):
    pass


class UserExceptionError(SMSClientError):
    pass


class UserCommentExceptionError(SMSClientError):
    pass



import datetime

import pytest

from pysmscenter.exceptions import SMSClientError
from pysmscenter.utils import bool2str, parse_date, raise_for_errors, ts2epoch


def test_raise_for_errors_does_not_raise_on_success():
    response = {"status": "1", "error": "0"}
    raise_for_errors(response, SMSClientError)


def test_raise_for_errors_raises_with_code_and_message():
    response = {
        "status": "0",
        "error": "123",
        "remarks": "error message",
    }

    with pytest.raises(SMSClientError) as exc:
        raise_for_errors(response, SMSClientError)

    assert exc.value.code == "123"
    assert exc.value.message == "error message"
    assert exc.value.response == response


def test_parse_date_valid():
    d = parse_date("2024-01-05")
    assert d.year == 2024
    assert d.month == 1
    assert d.day == 5


@pytest.mark.parametrize(
    "value",
    [
        "2024/01/05",
        "05-01-2024",
        "20240105",
        "Jan 5 2024",
    ],
)
def test_parse_date_invalid_format(value):
    with pytest.raises(ValueError, match="Invalid date format"):
        parse_date(value)


def test_parse_date_invalid_date():
    with pytest.raises(ValueError, match="Invalid date format"):
        parse_date("2024-02-31")


@pytest.mark.parametrize(
    ("value", "expected"),
    [
        (True, "true"),
        (False, "false"),
        (None, None),
    ],
)
def test_bool2str(value, expected):
    assert bool2str(value) == expected


def test_ts2epoch_with_int():
    assert ts2epoch(1700000000) == 1700000000


def test_ts2epoch_with_aware_datetime():
    dt = datetime.datetime(2024, 1, 1, tzinfo=datetime.UTC)
    assert ts2epoch(dt) == int(dt.timestamp())


def test_ts2epoch_with_naive_datetime_assumes_utc():
    dt = datetime.datetime(2024, 1, 1)  # noqa: DTZ001
    epoch = ts2epoch(dt)
    assert epoch == int(dt.replace(tzinfo=datetime.UTC).timestamp())



import datetime
import re
from typing import Any

from pysmscenter.exceptions import SMSClientError


def raise_for_errors(
    response: dict[str, Any],
    exc: type[SMSClientError],
) -> None:
    if response.get("status") == "0":
        code = str(response.get("error", "")) or None
        remarks = response.get("remarks", "") or None
        raise exc(message=remarks, code=code, response=response)


_DATE_RE = re.compile(r"^\d{4}-\d{2}-\d{2}$")


def parse_date(value: str) -> datetime.date:
    if not _DATE_RE.fullmatch(value):
        raise ValueError(f"Invalid date format: {value}. Expected YYYY-MM-DD")

    try:
        return datetime.date.fromisoformat(value)
    except ValueError as exc:
        raise ValueError(f"Invalid date format: {value}. Expected YYYY-MM-DD") from exc


def bool2str(value: bool | None) -> str | None:
    if value is None:
        return None
    return "true" if value else "false"


def ts2epoch(value: int | datetime.datetime) -> int:
    if isinstance(value, int):
        return value
    if value.tzinfo is None:
        value = value.replace(tzinfo=datetime.UTC)
    return int(value.timestamp())



from typing import cast

from pysmscenter.types import PurchaseRawResponse

from .manager import Manager


class PurchaseManager(Manager):
    name = "purchase"

    def __str__(self) -> str:
        return self.__class__.__name__

    def list(self) -> PurchaseRawResponse:
        """List available purchase options.

        Returns:
            PurchaseRawResponse: Response from the API.
        """
        response = self.call("GET", "purchase/list")

        return cast(PurchaseRawResponse, response)



from typing import Any
from urllib.parse import urljoin

import pytest
from urllib3.util.retry import Retry

from pysmscenter import SMSClient
from pysmscenter.exceptions import CredentialError
from pysmscenter.main import BaseHTTPClient, SMSAuthClient


@pytest.fixture
def auth_client_fixture(mocker: Any) -> tuple[Any, Any]:
    auth_client = mocker.Mock()
    auth_ctor = mocker.patch("pysmscenter.main.SMSAuthClient", return_value=auth_client)
    return auth_client, auth_ctor


class TestSMSClient:
    def test_init_requires_api_key(self) -> None:
        with pytest.raises(CredentialError, match="API key is required"):
            SMSClient("")

    def test_repr(self, client: SMSClient) -> None:
        assert repr(client) == f"<SMSClient base_url={SMSClient.BASE_URL!r}>"

    def test_session_creates_and_reuses_session(self, mocker: Any) -> None:
        mock_session = mocker.Mock()
        session_ctor = mocker.patch("pysmscenter.main.Session", return_value=mock_session)
        adapter = mocker.Mock()
        adapter_ctor = mocker.patch("pysmscenter.main.HTTPAdapter", return_value=adapter)

        client = SMSClient("test-api-key", max_retries=2)

        session_one = client.session
        session_two = client.session

        assert session_one is mock_session
        assert session_two is mock_session
        session_ctor.assert_called_once_with()
        adapter_ctor.assert_called_once()
        retry_arg = adapter_ctor.call_args.kwargs.get("max_retries")
        assert isinstance(retry_arg, Retry)
        mock_session.mount.assert_any_call("http://", adapter)
        mock_session.mount.assert_any_call("https://", adapter)
        assert mock_session.mount.call_count == 2

    def test_session_raises_when_closed(self, client: SMSClient) -> None:
        client.close()

        with pytest.raises(RuntimeError, match="Session is closed"):
            _ = client.session

    def test_close_closes_session(self, client: SMSClient, mocker: Any) -> None:
        mock_session = mocker.Mock()
        client._session = mock_session

        client.close()

        mock_session.close.assert_called_once_with()
        assert client._session is mock_session
        assert client._closed is True

    def test_context_manager_closes_session(self, mocker: Any) -> None:
        client = SMSClient("test-api-key")
        mock_session = mocker.Mock()
        client._session = mock_session

        result = client.__enter__()
        assert result is client

        exit_result = client.__exit__(None, None, None)
        assert exit_result is False
        mock_session.close.assert_called_once_with()
        assert client._session is mock_session
        assert client._closed is True

    def test_setup_managers(self, client: SMSClient) -> None:
        for manager_cls in SMSClient.managers:
            manager = getattr(client, manager_cls.name)
            assert isinstance(manager, manager_cls)
            assert manager.client is client

    def test_fetch_data_sets_defaults_and_calls_request(self, client: SMSClient, mocker: Any) -> None:
        response_json = {
            "status": "1",
            "balance": "2",
            "remarks": "Success",
            "error": "0",
        }
        response = mocker.Mock()
        response.json.return_value = response_json
        mock_session = mocker.Mock()
        mock_session.request.return_value = response
        client._session = mock_session

        result = client.fetch_data("GET", "balance")

        mock_session.request.assert_called_once_with(
            "GET",
            urljoin(SMSClient.BASE_URL, "balance"),
            params={
                "type": SMSClient.DEFAULT_TYPE,
                "key": "test-api-key",
            },
            timeout=client.timeout,
        )
        response.raise_for_status.assert_called_once_with()
        assert result == response_json

    def test_fetch_data_overrides_key_and_preserves_type(self, client: SMSClient, mocker: Any) -> None:
        response_json = {
            "status": "1",
            "balance": "2",
            "remarks": "Success",
            "error": "0",
        }
        response = mocker.Mock()
        response.json.return_value = response_json
        mock_session = mocker.Mock()
        mock_session.request.return_value = response
        client._session = mock_session

        result = client.fetch_data(
            "GET",
            "mobile/check",
            params={"type": "xml", "foo": "bar", "key": "override"},
        )

        mock_session.request.assert_called_once_with(
            "GET",
            urljoin(SMSClient.BASE_URL, "mobile/check"),
            params={
                "type": "xml",
                "foo": "bar",
                "key": "test-api-key",
            },
            timeout=client.timeout,
        )
        response.raise_for_status.assert_called_once_with()
        assert result == response_json

    def test_fetch_data_raises_credential_error(self, client: SMSClient, mocker: Any) -> None:
        response_json = {"status": 0, "error": 101, "remarks": "Invalid API key"}
        response = mocker.Mock()
        response.json.return_value = response_json
        mock_session = mocker.Mock()
        mock_session.request.return_value = response
        client._session = mock_session

        with pytest.raises(CredentialError, match="Invalid API key") as exc:
            client.fetch_data("GET", "balance")

        assert exc.value.code == "101"
        assert exc.value.message == "Invalid API key"
        assert exc.value.response == response_json

    def test_from_credentials_uses_auth_client(self, auth_client_fixture: tuple[Any, Any]) -> None:
        auth_client, auth_ctor = auth_client_fixture
        auth_client.get_key.return_value = {
            "status": "1",
            "remarks": "Success",
            "key": "fake_api_key_123456",
            "error": "0",
        }

        client = SMSClient.from_credentials("user", "pass", max_retries=2)

        kwargs = auth_ctor.call_args.kwargs
        assert kwargs["max_retries"] == 2
        assert kwargs["timeout"] == SMSClient.DEFAULT_TIMEOUT
        auth_client.get_key.assert_called_once_with("user", "pass")
        auth_client.close.assert_called_once_with()
        assert client.api_key == "fake_api_key_123456"

    def test_from_credentials_raises_when_missing_key(self, auth_client_fixture: tuple[Any, Any]) -> None:
        auth_client, auth_ctor = auth_client_fixture
        auth_client.get_key.return_value = {
            "status": "1",
            "balance": "2",
            "remarks": "Error: Check your credentials",
            "error": "101",
        }

        with pytest.raises(CredentialError, match="Failed to retrieve API key"):
            SMSClient.from_credentials("user", "pass")

        auth_ctor.assert_called_once_with(max_retries=0, timeout=SMSClient.DEFAULT_TIMEOUT)
        auth_client.close.assert_called_once_with()

    def test_reset_api_key_returns_new_key(self, auth_client_fixture: tuple[Any, Any]) -> None:
        auth_client, auth_ctor = auth_client_fixture
        auth_client.reset_key.return_value = {
            "status": "1",
            "remarks": "Success",
            "key": "new_fake_api_key_654321",
            "error": "0",
        }

        result = SMSClient.reset_api_key("user", "pass", max_retries=3)

        auth_ctor.assert_called_once_with(max_retries=3, timeout=SMSClient.DEFAULT_TIMEOUT)
        auth_client.reset_key.assert_called_once_with("user", "pass")
        auth_client.close.assert_called_once_with()
        assert result == "new_fake_api_key_654321"

    def test_reset_api_key_raises_when_missing_key(self, auth_client_fixture: tuple[Any, Any]) -> None:
        auth_client, auth_ctor = auth_client_fixture
        auth_client.reset_key.return_value = {
            "status": "1",
            "balance": "2",
            "remarks": "Error: Check your credentials",
            "error": "101",
        }

        with pytest.raises(CredentialError, match="Failed to reset API key"):
            SMSClient.reset_api_key("user", "pass")

        auth_ctor.assert_called_once_with(max_retries=0, timeout=SMSClient.DEFAULT_TIMEOUT)
        auth_client.close.assert_called_once_with()


class TestBaseHTTPClient:
    def test_repr(self) -> None:
        client = BaseHTTPClient()
        assert repr(client) == f"<BaseHTTPClient base_url={BaseHTTPClient.BASE_URL!r}>"

    def test_build_params_defaults_type(self) -> None:
        client = BaseHTTPClient()

        result = client._build_params({"foo": "bar"})

        assert result == {"foo": "bar", "type": BaseHTTPClient.DEFAULT_TYPE}

    def test_request_uses_base_url_and_timeout(self, mocker: Any) -> None:
        client = BaseHTTPClient(timeout=(1.0, 2.0))
        mock_session = mocker.Mock()
        client._session = mock_session
        response = mocker.Mock()
        response.json.return_value = {
            "status": "1",
            "balance": "2",
            "remarks": "Success",
            "error": "0",
        }
        mock_session.request.return_value = response

        result = client._request("GET", "health", params={"foo": "bar"})

        mock_session.request.assert_called_once_with(
            "GET",
            urljoin(BaseHTTPClient.BASE_URL, "health"),
            params={"foo": "bar", "type": BaseHTTPClient.DEFAULT_TYPE},
            timeout=client.timeout,
        )
        response.raise_for_status.assert_called_once_with()
        assert result == {
            "status": "1",
            "balance": "2",
            "remarks": "Success",
            "error": "0",
        }


class TestSMSAuthClient:
    def test_get_key(self, mocker: Any) -> None:
        client = SMSAuthClient()
        request_mock = mocker.patch.object(
            client,
            "_request",
            return_value={
                "status": "1",
                "remarks": "Success",
                "key": "fake_api_key_123456",
                "error": "0",
            },
        )

        result = client.get_key("user", "pass")

        request_mock.assert_called_once_with("GET", "key/get", params={"username": "user", "password": "pass"})
        assert result == {
            "status": "1",
            "remarks": "Success",
            "key": "fake_api_key_123456",
            "error": "0",
        }

    def test_reset_key(self, mocker: Any) -> None:
        client = SMSAuthClient()
        request_mock = mocker.patch.object(
            client,
            "_request",
            return_value={
                "status": "1",
                "remarks": "Success",
                "key": "new_fake_api_key_654321",
                "error": "0",
            },
        )

        result = client.reset_key("user", "pass")

        request_mock.assert_called_once_with("GET", "key/reset", params={"username": "user", "password": "pass"})
        assert result == {
            "status": "1",
            "remarks": "Success",
            "key": "new_fake_api_key_654321",
            "error": "0",
        }



from typing import cast, overload

from pysmscenter.exceptions import GroupExceptionError
from pysmscenter.types import BaseResponse, GroupAddContactData, GroupData, GroupGetData, GroupListData
from pysmscenter.utils import raise_for_errors

from .manager import Manager


class GroupManager(Manager):
    name = "group"

    def __str__(self) -> str:
        return self.__class__.__name__

    def add(self, name: str) -> GroupData:
        """
        Add a new group.

        Args:
            name (str): The name of the group to be added.
        """

        response = self.call("GET", "group/add", {"name": name})
        raise_for_errors(response, GroupExceptionError)
        return cast(GroupData, response)

    def delete(self, group_id: str) -> GroupData:
        """
        Delete a group.

        Args:
            group_id (str): The ID of the group to be deleted.
        """

        response = self.call("GET", "group/delete", {"groupId": group_id})
        raise_for_errors(response, GroupExceptionError)

        return cast(GroupData, response)

    def list(self) -> GroupListData:
        """
        List all groups.

        Returns:
            GroupData: Response from the API.
        """

        response = self.call("GET", "group/list")

        return cast(GroupListData, response)

    def get(self, group_id: str) -> GroupGetData:
        """
        Get details of a specific group.

        Args:
            group_id (str): The ID of the group to retrieve.

        Raises:
            GroupExceptionError: If the API response indicates an error.

        Returns:
            GroupGetData: Response from the API.
        """

        response = self.call("GET", "group/get", {"groupId": group_id})

        raise_for_errors(response, GroupExceptionError)

        return cast(GroupGetData, response)

    def add_contact(self, group_id: str, contact_id: str) -> GroupAddContactData:
        """
        Add a contact to a group.

        Args:
            group_id (str): The ID of the group to which the contact will be added.
            contact_id (str): The ID of the contact to be added.

        Raises:
            GroupExceptionError: If the API response indicates an error.

        Returns:
            GroupAddContactData: Response from the API.
        """
        params = {
            "groupId": group_id,
            "contactId": contact_id,
        }

        response = self.call("GET", "group/addContact", params)

        raise_for_errors(response, GroupExceptionError)

        return cast(GroupAddContactData, response)

    @overload
    def delete_contact(self, *, group_id: str, contact_id: str) -> BaseResponse: ...

    @overload
    def delete_contact(self, *, contact_group_id: str) -> BaseResponse: ...

    def delete_contact(
        self,
        *,
        group_id: str | None = None,
        contact_id: str | None = None,
        contact_group_id: str | None = None,
    ) -> BaseResponse:
        """
        Delete a contact from a group.

        Args:
            group_id (str, optional): The ID of the group from which the contact will be deleted. Required
            if contact_group_id is not provided.
            contact_id (str, optional): The ID of the contact to be deleted. Required
            if contact_group_id is not provided.
            contact_group_id (str, optional): The ID of the contact-group link to be deleted. Required
            if group_id and contact_id are not provided.
        Raises:
            GroupExceptionError: If the API response indicates an error.
            ValueError: If neither contact_group_id nor both group_id and contact_id are provided.
        Returns:
            BaseResponse: Response from the API.
        """
        error_message = "Either contact_group_id or both group_id and contact_id must be provided."

        has_group_and_contact = group_id is not None and contact_id is not None
        has_contact_group_id = contact_group_id is not None
        has_any_group_contact = group_id is not None or contact_id is not None

        if not has_group_and_contact and not has_contact_group_id:
            raise ValueError(error_message)

        if has_contact_group_id and has_any_group_contact:
            raise ValueError(error_message)

        if contact_group_id is not None:
            params = {"contactGroupId": contact_group_id}
        else:
            params = {
                "groupId": group_id,
                "contactId": contact_id,
            }

        response = self.call("GET", "group/deleteContact", params)

        raise_for_errors(response, GroupExceptionError)

        return cast(BaseResponse, response)

    def delete_all_contacts(self, group_id: str) -> BaseResponse:
        """
        Delete all contacts from a group.

        Args:
            group_id (str): The ID of the group from which all contacts will be deleted.
        Raises:
            GroupExceptionError: If the API response indicates an error.
        Returns:
            BaseResponse: Response from the API.
        """
        response = self.call("GET", "group/deleteAllContacts", {"groupId": group_id})

        raise_for_errors(response, GroupExceptionError)

        return cast(BaseResponse, response)



from typing import cast

from pysmscenter.exceptions import HLRExceptionError
from pysmscenter.types import HLRLookupRawResponse
from pysmscenter.utils import raise_for_errors

from .manager import Manager


class HLRManager(Manager):
    name = "hlr"

    def __str__(self) -> str:
        return self.__class__.__name__

    def lookup(self, mobile: str) -> HLRLookupRawResponse:
        """
        Perform an HLR lookup for the given mobile number.

        Args:
            mobile (str): The mobile number to look up.

        Returns:
            HLRLookupRawResponse: An object containing the HLR lookup result.
        """
        # Implement the HLR lookup logic here
        response = self.call("GET", "hlr/lookup", params={"mobile": mobile})
        raise_for_errors(response, HLRExceptionError)
        return cast(HLRLookupRawResponse, response)



from .balance_manager import BalanceManager
from .contact_manager import ContactManager
from .group_manager import GroupManager
from .history_manager import HistoryManager
from .hlr_manager import HLRManager
from .mobile_manager import MobileManager
from .purchase_manager import PurchaseManager
from .sms_manager import SmsManager
from .status_manager import StatusManager
from .two_factor_manager import TwoFactorManager

__all__ = [
    "BalanceManager",
    "ContactManager",
    "GroupManager",
    "HLRManager",
    "HistoryManager",
    "MobileManager",
    "PurchaseManager",
    "SmsManager",
    "StatusManager",
    "TwoFactorManager",
]



from .base import BaseResponse


class TwoFactorRawResponse(BaseResponse, total=False):
    authId: str  #  noqa: N815
    authStatus: str  #  noqa: N815


class TwoFactorCheckResponse(BaseResponse, total=False):
    auth: str



import types
from collections.abc import Mapping
from typing import Any, ClassVar, Self, cast
from urllib.parse import urljoin

from requests import Session
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry

from pysmscenter.managers.balance_manager import BalanceManager
from pysmscenter.managers.contact_manager import ContactManager
from pysmscenter.managers.group_manager import GroupManager
from pysmscenter.managers.history_manager import HistoryManager
from pysmscenter.managers.hlr_manager import HLRManager
from pysmscenter.managers.manager import Manager
from pysmscenter.managers.mobile_manager import MobileManager
from pysmscenter.managers.purchase_manager import PurchaseManager
from pysmscenter.managers.sms_manager import SmsManager
from pysmscenter.managers.status_manager import StatusManager
from pysmscenter.managers.two_factor_manager import TwoFactorManager
from pysmscenter.managers.user_manager import UserManager
from pysmscenter.types.key_types import KeyRawResponse

from .exceptions import CredentialError

type Timeout = tuple[float, float]


class BaseHTTPClient:
    BASE_URL: str = "https://smscenter.gr/api/"
    DEFAULT_TYPE: str = "json"
    DEFAULT_TIMEOUT: ClassVar[Timeout] = (5.0, 30.0)

    def __init__(
        self, max_retries: int = 0, timeout: Timeout | None = DEFAULT_TIMEOUT, backoff_factor: float = 0.5
    ) -> None:
        self.max_retries = max_retries
        self.timeout = timeout
        self.backoff_factor = backoff_factor
        self._session: Session = self._build_session()
        self._closed: bool = False

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__} base_url={self.BASE_URL!r}>"

    @property
    def session(self) -> Session:
        if self._closed:
            raise RuntimeError("Session is closed")
        return self._session

    def close(self) -> None:
        if not self._closed:
            self._session.close()
            self._closed = True

    def __enter__(self) -> Self:
        if self._closed:
            raise RuntimeError("Cannot enter context with closed session")
        return self

    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        traceback: types.TracebackType | None,
    ) -> bool:
        self.close()
        return False

    def _build_session(self) -> Session:
        session = Session()

        retry_strategy = Retry(
            total=self.max_retries,
            backoff_factor=self.backoff_factor,
            status_forcelist=[429, 500, 502, 503, 504],
            allowed_methods=None,
            raise_on_status=False,
        )
        adapter = HTTPAdapter(max_retries=retry_strategy)
        session.mount("http://", adapter)
        session.mount("https://", adapter)
        return session

    def _build_params(self, params: Mapping[str, Any] | None = None) -> dict[str, Any]:
        params_dict = dict(params) if params is not None else {}
        params_dict.setdefault("type", self.DEFAULT_TYPE)
        return params_dict

    def _request(self, method: str, endpoint: str, params: Mapping[str, Any] | None = None) -> dict[str, Any]:
        url = urljoin(self.BASE_URL, endpoint)
        request_params = self._build_params(params)
        method = method.upper()
        response = self.session.request(method, url, params=request_params, timeout=self.timeout)
        response.raise_for_status()
        return response.json()


class SMSAuthClient(BaseHTTPClient):
    def get_key(self, username: str, password: str) -> KeyRawResponse:
        """
        Get an API key for the given username and password.

        Args:
            username (str): The username to authenticate with.
            password (str): The password to authenticate with.
        Returns:
            KeyRawResponse: The API key for the authenticated user.
        """
        response = self._request("GET", "key/get", params={"username": username, "password": password})
        return cast(KeyRawResponse, response)

    def reset_key(self, username: str, password: str) -> KeyRawResponse:
        """
        Reset the API key for the authenticated user.

        Returns:
            KeyRawResponse: The new API key for the authenticated user.
        """

        response = self._request("GET", "key/reset", params={"username": username, "password": password})
        return cast(KeyRawResponse, response)


class SMSClient(BaseHTTPClient):
    mobile: "MobileManager"
    sms: "SmsManager"
    balance: "BalanceManager"
    history: "HistoryManager"
    status: "StatusManager"
    contact: "ContactManager"
    group: "GroupManager"
    purchase: "PurchaseManager"
    hlr: "HLRManager"
    two_factor: "TwoFactorManager"
    user: "UserManager"

    managers: ClassVar[list[type[Manager]]] = [
        MobileManager,
        SmsManager,
        BalanceManager,
        HistoryManager,
        StatusManager,
        ContactManager,
        GroupManager,
        PurchaseManager,
        HLRManager,
        TwoFactorManager,
        UserManager,
    ]

    def __init__(
        self, api_key: str, max_retries: int = 0, timeout: Timeout | None = BaseHTTPClient.DEFAULT_TIMEOUT
    ) -> None:
        super().__init__(max_retries=max_retries, timeout=timeout)
        self.api_key = api_key
        if not api_key:
            raise CredentialError("API key is required")

        self._setup_managers()

    @classmethod
    def from_credentials(
        cls,
        username: str,
        password: str,
        max_retries: int = 0,
        timeout: Timeout | None = BaseHTTPClient.DEFAULT_TIMEOUT,
    ) -> "SMSClient":
        """
        Create an SMSClient instance from a username and password.

        Args:
            username (str): The username to authenticate with.
            password (str): The password to authenticate with.
            max_retries (int, optional): The maximum number of retries for HTTP requests. Defaults to 0.
            timeout (Timeout, optional): The timeout for HTTP requests. Defaults to BaseHTTPClient.DEFAULT_TIMEOUT.
        Returns:
            SMSClient: An instance of SMSClient authenticated with the provided credentials.
        """
        auth_client = SMSAuthClient(max_retries=max_retries, timeout=timeout)
        try:
            key_response = auth_client.get_key(username, password)
        finally:
            auth_client.close()

        api_key = key_response.get("key")
        if not api_key:
            raise CredentialError("Failed to retrieve API key with provided credentials")
        return cls(api_key=api_key, max_retries=max_retries, timeout=timeout)

    @classmethod
    def reset_api_key(
        cls,
        username: str,
        password: str,
        max_retries: int = 0,
        timeout: Timeout | None = BaseHTTPClient.DEFAULT_TIMEOUT,
    ) -> str:
        auth_client = SMSAuthClient(max_retries=max_retries, timeout=timeout)
        try:
            key_response = auth_client.reset_key(username, password)
        finally:
            auth_client.close()

        new_api_key = key_response.get("key")
        if not new_api_key:
            raise CredentialError("Failed to reset API key with provided credentials")

        return new_api_key

    def _setup_managers(self) -> None:
        for manager in self.managers:
            setattr(self, manager.name, manager(self))

    def fetch_data(
        self,
        method: str,
        endpoint: str,
        params: Mapping[str, Any] | None = None,
    ) -> dict[str, Any]:
        response_json = self._request(method, endpoint, params=params)
        self._raise_for_credential_error(response_json)
        return response_json

    def _build_params(self, params: Mapping[str, Any] | None = None) -> dict[str, Any]:
        params_dict = super()._build_params(params)
        params_dict.update({"key": self.api_key})
        return params_dict

    @staticmethod
    def _raise_for_credential_error(response_json: Mapping[str, Any]) -> None:
        if str(response_json.get("status")) == "0" and str(response_json.get("error")) == "101":
            raise CredentialError(
                response_json.get("remarks"),
                code=str(response_json.get("error")),
                response=dict(response_json),
            )



from collections.abc import Mapping
from typing import Any, ClassVar, TYPE_CHECKING

if TYPE_CHECKING:
    from pysmscenter.main import SMSClient


class Manager:
    name: ClassVar[str]

    def __init__(self, client: "SMSClient") -> None:
        self.client = client

    def call(
        self,
        method: str,
        endpoint: str,
        params: Mapping[str, Any] | None = None,
    ) -> dict[str, Any]:
        return self.client.fetch_data(method, endpoint, params)



from collections.abc import Sequence
from typing import cast

from pysmscenter.exceptions import SMSExceptionError
from pysmscenter.types import SMSBulkRawData, SMSCancelRawData, SMSRawData, Timestamp
from pysmscenter.utils import bool2str, raise_for_errors, ts2epoch

from .manager import Manager


class SmsManager(Manager):
    name = "sms"

    def __str__(self) -> str:
        return self.__class__.__name__

    def send(
        self,
        to: str,
        text: str,
        sender: str,
        ucs: bool | None = None,
        flash: bool | None = None,
        timestamp: Timestamp | None = None,
        callback: str | None = None,
    ) -> SMSRawData:
        """Send an SMS.

        Args:
            to (str): Mobile number to send the sms to
            text (str): Text of the sms to send
            sender (str): Sender of the sms
            ucs (bool, optional): Whether the sms is unicode. Defaults to None.
            flash (bool, optional): Whether the sms is flash. Defaults to None.
            timestamp (Timestamp, optional): Timestamp for scheduled sending. Defaults to None.
            callback (str, optional): Callback URL for delivery reports. Defaults to None.

        Raises:
            SMSExceptionError: If the API response indicates an error.

        Returns:
            SMSRawData: Response from the API.
        """
        params = {
            "to": to,
            "text": text,
            "from": sender,
            "ucs": bool2str(ucs) if ucs is not None else None,
            "flash": bool2str(flash) if flash is not None else None,
            "timestamp": ts2epoch(timestamp) if timestamp is not None else None,
            "callback": callback,
        }
        params = {key: value for key, value in params.items() if value is not None}

        response = self.call("GET", "sms/send", params)

        raise_for_errors(response, SMSExceptionError)

        return cast(SMSRawData, response)

    def bulk(
        self,
        to: Sequence[str] | str,
        text: str,
        sender: str,
        ucs: bool | None = None,
        flash: bool | None = None,
        timestamp: Timestamp | None = None,
    ) -> SMSBulkRawData:
        """Send an SMS to multiple recipients.

        Args:
            to (Sequence[str] | str): multiple mobiles to send the sms to
            text (str): Text of the sms to send
            sender (str): Sender of the sms
            ucs (bool, optional): Whether the sms is unicode. Defaults to None.
            flash (bool, optional): Whether the sms is flash. Defaults to None.
            timestamp (Timestamp, optional): Timestamp for scheduled sending. Defaults to None.

        Raises:
            SMSExceptionError: If the API response indicates an error.

        Returns:
            SMSBulkRawData: Response from the API.
        """

        if isinstance(to, list | tuple):
            to = ",".join(to)

        params = {
            "to": to,
            "text": text,
            "from": sender,
            "ucs": bool2str(ucs) if ucs is not None else None,
            "flash": bool2str(flash) if flash is not None else None,
            "timestamp": ts2epoch(timestamp) if timestamp is not None else None,
        }
        params = {key: value for key, value in params.items() if value is not None}

        response = self.call("GET", "sms/bulk", params)
        raise_for_errors(response, SMSExceptionError)

        return cast(SMSBulkRawData, response)

    def cancel(self, sms_id: str) -> SMSCancelRawData:
        """Cancel a scheduled SMS.

        Note:
            Only scheduled (future timestamp) messages can be canceled.
            Immediate messages are usually not cancellable.

        Args:
            sms_id (str): ID of the sms to cancel

        Raises:
            SMSExceptionError: If the API response indicates an error.

        Returns:
            SMSCancelRawData: Response from the API.
        """
        params = {"smsId": sms_id}
        response = self.call("GET", "sms/cancel", params)
        raise_for_errors(response, SMSExceptionError)
        return cast(SMSCancelRawData, response)



from typing import TypedDict

from .base import BaseResponse


class PurchaseItem(TypedDict):
    purchaseId: str
    timestamp: str
    cost: str
    sms: str


class PurchaseRawResponse(BaseResponse, total=False):
    purchases: list[PurchaseItem]



import datetime
from typing import TypedDict

from pysmscenter.types.base import BaseResponse

type Timestamp = int | datetime.datetime


class SMSRawData(BaseResponse):
    id: str
    cost: str
    balance: str
    mcc: str
    mnc: str


class SMSCancelRawData(BaseResponse, total=False):
    smsId: str  # noqa: N815


class SMSBulkItem(TypedDict, total=False):
    id: str
    smsId: str
    msisdn: str


class SMSBulkRawData(BaseResponse, total=False):
    id: list[str]
    sms: list[SMSBulkItem]
    balance: str
    cost: str
    accepted: str
    rejected: str



from typing import TypedDict

from pysmscenter.types.base import BaseResponse


class MobileData(TypedDict):
    cost: int
    country: str
    countryCode: int
    gsmCode: str
    mcc: str
    mnc: str
    msisdn: str
    national: str
    number: str


class MobileRawData(BaseResponse, total=False):
    total: int
    mobile: MobileData



from pysmscenter.types.balance_types import BalanceRawData
from pysmscenter.types.base import BaseResponse
from pysmscenter.types.contact_types import (
    Contact,
    ContactData,
    ContactDetail,
    ContactID,
    ContactListData,
    Contacts,
    DateLike,
)
from pysmscenter.types.group_types import (
    Group,
    GroupAddContactData,
    GroupAddContactGroup,
    GroupContact,
    GroupContactLink,
    GroupData,
    GroupGetData,
    GroupGetGroup,
    GroupList,
    GroupListData,
)
from pysmscenter.types.history_types import (
    GroupListHistoryRawResponse,
    HistoryGroupItem,
    HistoryGroupSms,
    SingleListHistoryRawData,
    SmS,
)
from pysmscenter.types.hlr_types import HLRLookupRawResponse
from pysmscenter.types.key_types import KeyRawResponse
from pysmscenter.types.mobile_types import MobileData, MobileRawData
from pysmscenter.types.purchase_types import PurchaseItem, PurchaseRawResponse
from pysmscenter.types.sms_types import SMSBulkRawData, SMSCancelRawData, SMSRawData, Timestamp
from pysmscenter.types.status_types import StatusRawResponse
from pysmscenter.types.two_factor_types import TwoFactorCheckResponse, TwoFactorRawResponse
from pysmscenter.types.user_types import (
    UserCommentListRawResponseType,
    UserCommentRawResponse,
    UserListRawResponseType,
    UserRawResponse,
)

__all__ = [
    "BalanceRawData",
    "BaseResponse",
    "Contact",
    "ContactData",
    "ContactDetail",
    "ContactID",
    "ContactListData",
    "Contacts",
    "DateLike",
    "Group",
    "GroupAddContactData",
    "GroupAddContactGroup",
    "GroupContact",
    "GroupContactLink",
    "GroupData",
    "GroupGetData",
    "GroupGetGroup",
    "GroupList",
    "GroupListData",
    "GroupListHistoryRawResponse",
    "HLRLookupRawResponse",
    "HistoryGroupItem",
    "HistoryGroupSms",
    "KeyRawResponse",
    "MobileData",
    "MobileRawData",
    "PurchaseItem",
    "PurchaseRawResponse",
    "SMSBulkRawData",
    "SMSCancelRawData",
    "SMSRawData",
    "SingleListHistoryRawData",
    "SmS",
    "StatusRawResponse",
    "Timestamp",
    "TwoFactorCheckResponse",
    "TwoFactorRawResponse",
    "UserCommentListRawResponseType",
    "UserCommentRawResponse",
    "UserListRawResponseType",
    "UserRawResponse",
]



from typing import cast

from pysmscenter.types import BalanceRawData

from .manager import Manager


class BalanceManager(Manager):
    name = "balance"

    def __str__(self) -> str:
        return self.__class__.__name__

    def check(self) -> BalanceRawData:
        """Check the account balance.

        Returns:
            BalanceRawData: Response from the API.
        """
        response = self.call("GET", "me/balance")

        return cast(BalanceRawData, response)



from typing import cast

from pysmscenter.types import GroupListHistoryRawResponse, SingleListHistoryRawData

from .manager import Manager


class HistoryManager(Manager):
    name = "history"

    def __str__(self) -> str:
        return self.__class__.__name__

    def group_list(self) -> GroupListHistoryRawResponse:
        """
        Get the grouped SMS history list.
        Returns:
            GroupListHistoryRawResponse: Response from the API.

        """
        response = self.call("GET", "history/group/list")
        return cast(GroupListHistoryRawResponse, response)

    def single_list(self) -> SingleListHistoryRawData:
        """Get the single (non-grouped) SMS history list.

        Returns:
            SingleListHistoryRawData: Response from the API.
        """
        response = self.call("GET", "history/single/list")

        return cast(SingleListHistoryRawData, response)



from typing import cast

from pysmscenter.types import StatusRawResponse

from .manager import Manager


class StatusManager(Manager):
    name = "status"

    def __str__(self) -> str:
        return self.__class__.__name__

    def get(self) -> StatusRawResponse:
        """Get delivery statuses for recent messages.

        Returns:
            StatusRawResponse: Response from the API.
        """
        response = self.call("GET", "status/get")
        return cast(StatusRawResponse, response)

    def sms(self, sms_id: str) -> StatusRawResponse:
        """Get delivery status for a specific SMS.

        Args:
            sms_id: SMS ID to look up.

        Returns:
            StatusRawResponse: Response from the API.
        """
        response = self.call("GET", "status/sms", {"smsId": sms_id})
        return cast(StatusRawResponse, response)
